#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .exception import JiraApiException, JiraValidationException


class Issue:

    __module__ = "jira_yak"

    def __init__(self, api, key):
        self.client = api
        self.project_key = key
        self.filter = self.client.filter

    def get_issues_from_filter(self, filter_id):
        """Get Jira issues returned by a given JQL filter_id.

        :param filter_id: id for the filter
        :return:
        """
        if not filter_id or filter_id is None:
            raise IssueValidationException("[*] Invalid filter_id")

        jql = self.filter.get_jql_filter(filter_id).jql
        issues = None
        try:
            issues = self.client.search_issues(jql, maxResults=100)
        except IssueException:
            print("[!] Failed to get issues from Jira filter.\nRetrying in a few seconds.")
            try:
                issues = self.client.search_issues(jql, maxResults=100)
            except IssueException:
                print("[!] Failed to get issues from Jira filter.")
        finally:
            return issues

    def get_issues(self, filter_id):
        """Seems weirdly like a dupe of get_issues_from_filter(), but if things are working I don't wanna break it right now..

        :param filter_id: id for the filter
        :return:
        """
        if not filter_id or filter_id is None:
            raise IssueValidationException("[*] Invalid issue_key")

        issues = None
        try:
            issues = self.get_issues_from_filter(filter_id)
        except IssueException:
            print("[!] Failed to get issues from Jira filter.\nRetrying in a few seconds.")
            try:
                issues = self.get_issues_from_filter(filter_id)
            except IssueException:
                print("[!] Failed to get issues from Jira filter.")
        finally:
            return issues

    def get_issue(self, issue_key):
        """Get a Jira story given a key.

        :param issue_key: a Jira story key (such as AMB-1234)
        :return:
        """
        if not issue_key or issue_key is None:
            raise IssueValidationException("[*] Invalid issue_key")

        issue = None
        try:
            issue = self.client.issue(issue_key, fields="status", expand="changelog")
        except IssueException:
            print("[!] Failed to get Jira issue.\nRetrying in a few seconds.")
            try:
                issue = self.client.issue(issue_key, fields="status", expand="changelog")
            except IssueException:
                print("[!] Failed to get Jira issue.")
        finally:
            if issue is None:
                raise IssueException("[!] Unable to retrieve issue {}".format(issue_key))
            return issue

    @staticmethod
    def get_statuses(change_log):
        """Get all status changes from a change_log associated with a Jira story.

        :param change_log: expanded list of fields returned from Jira API for a single issue
        :return list: a list of dicts representing the status change and it's author
        """
        if not change_log or change_log is None:
            raise IssueValidationException("[*] change_log cannot be None")
        return [dict(
            authorName  = ch.author.displayName,
            created     = ch.created,
            fromString  = ch.items[0].fromString,
            toString    = ch.items[0].toString
        ) for ch in change_log if ch.items[0].field.lower() == "status"]

    def get_subtasks(self, issue_key):
        """Get subtasks of a Jira story.

        :param issue_key: a Jira story key (such as AMB-1234)
        :return list: a list of subtask names
        """
        if not issue_key or issue_key is None:
            raise IssueValidationException("[*] Invalid issue_key")

        issue = None
        try:
            issue = self.client.issue(issue_key, fields="status", expand="changelog")
        except IssueException:
            print("[!] Failed to get Jira issue.\nRetrying in a few seconds.")
            try:
                issue = self.client.issue(issue_key, fields="status", expand="changelog")
            except IssueException:
                print("[!] Failed to get Jira issue.")
        finally:
            if issue is None:
                raise IssueException("[!] NoneType response is not valid.")
            else:
                return [task.lower() for task in issue.fields.subtasks]

    def get_attachments(self, issue_key):
        """Get attachments from a Jira story.

        :param issue_key: a Jira story key (such as AMB-1234)
        :return list: a list of URLs pointing to story attachments
        """
        if not issue_key or issue_key is None:
            raise IssueValidationException("[*] Invalid issue_key")

        issue = None
        try:
            issue = self.client.issue(issue_key, fields="attachment", expand="changelog")
        except IssueException:
            print("[!] Failed to get Jira issue.\nRetrying in a few seconds.")
            try:
                issue = self.client.issue(issue_key, fields="attachment", expand="changelog")
            except IssueException:
                print("[!] Failed to get Jira issue.")
        finally:
            if issue is None:
                raise IssueException("[!] NoneType response is not valid.")
            else:
                return ["{}secure/attachment/{}/{}".format(self.client.host, a.id, a.filename) for a in issue.fields.attachment]

    def get_labels(self, issue_key):
        """Get a list of labels attached to a story by issue_key.

        :param issue_key: a Jira story key (such as AMB-1234)
        :return list: a list Jira story labels
        """
        if not issue_key or issue_key is None:
            raise IssueValidationException("[*] Invalid issue_key")

        issue = None
        try:
            issue = self.client.issue(issue_key, fields="labels", expand="changelog")
        except IssueException:
            print("[!] Failed to get Jira issue.\nRetrying in a few seconds.")
            try:
                issue = self.client.issue(issue_key, fields="labels", expand="changelog")
            except IssueException:
                print("[!] Failed to get Jira issue.")
        finally:
            if issue is None:
                raise IssueException("[!] NoneType response is not valid.")
            else:
                return [label.lower() for label in issue.fields.labels]


class IssueException(JiraApiException):
    pass


class IssueValidationException(JiraValidationException):
    pass
